#include <stdio.h>
#include <curses.h>

#include "world.h"
#include "color-pair.h"
#include "rule.h"
#include "mode.h"

#include "loop-output.h"

#define BOARD_OFFSET_X 2
#define BOARD_OFFSET_Y 2

#define PLAYER_OFFSET_X 13
#define PLAYER_OFFSET_Y 2

#define MONSTER_OFFSET_X 23
#define MONSTER_OFFSET_Y 0

#define MESSAGE_OFFSET_X 0
#define MESSAGE_OFFSET_Y 13

static int loop_output_board(world *wp);
static int loop_output_player(world *wp);
static int loop_output_monster(world *wp);
static int message_before_game(world *wp);
static int message_after_game(world *wp);
static int loop_output_message(world *wp);

static int
loop_output_board(world *wp)
{
  int i;
  int x;
  int y;
  int g;
  int attr;
  const char *s = NULL;

  if (wp->state == NULL)
    return 1;

  /* coordinate */
  attr = COLOR_PAIR(CPAIR_WHITE);
  attron(attr);

  /* assumes ASCII code */
  for (i = 0; i < 9; i++)
  {
    mvaddch(BOARD_OFFSET_Y - 2, BOARD_OFFSET_X + i, 'a' + i);
    mvaddch(BOARD_OFFSET_Y + i, BOARD_OFFSET_X - 2, '9' - i);
  }

  attroff(attr);

  /* grid */
  attr = COLOR_PAIR(CPAIR_WHITE);
  attron(attr);

  for (y = 0; y < 9; y++)
  {
    for (x = 0; x < 9; x++)
    {
      g = y * 9 + x;

      if (wp->state[18 + g] == -2)
        s = "x";
      else if ((x == 0) || (x == 8))
        s = ":";
      else
        s = ".";

      mvaddstr(BOARD_OFFSET_Y + 8 - y, BOARD_OFFSET_X + x, s);
    }
  }

  attroff(attr);

  /* piece */
  for (i = 0; i < 18; i++)
  {
    if (wp->state[i] < 0)
      continue;
    if (wp->state[i] >= 81)
    {
      fprintf(stderr, "loop_output_board: strange wp->state[%d] (%d)\n",
              i, wp->state[i]);
      continue;
    }

    x = wp->state[i] % 9;
    y = wp->state[i] / 9;

    if (i < 9)
    {
      s = "@";
      attr = COLOR_PAIR(CPAIR_CYAN);
    }
    else
    {
      s = "&";
      attr = COLOR_PAIR(CPAIR_RED);
    }

    if ((wp->grid_from >= 0) && (wp->state[i] == wp->grid_from))
      attr = COLOR_PAIR(CPAIR_YELLOW) | A_BOLD;

    if ((wp->last_move >= 0) && (wp->state[i] == wp->last_move))
      attr |= A_BOLD;

    attron(attr);    
    mvaddstr(BOARD_OFFSET_Y + 8 - y, BOARD_OFFSET_X + x, s);
    attroff(attr);
  }

  return 0;
}

static int
loop_output_player(world *wp)
{
  int i;
  int attr;
  char buf[256];

  if (wp == NULL)
    return 0;

  if ((wp->turn_player >= 0) && (wp->turn_player < 2))
  {
    if (wp->turn_player == 0)
      attr = COLOR_PAIR(CPAIR_CYAN);
    else
      attr = COLOR_PAIR(CPAIR_RED);

    attron(attr);
    mvaddstr(PLAYER_OFFSET_Y + 8 - wp->turn_player * 8,
             PLAYER_OFFSET_X + 0,
             "*");
    attroff(attr);
  }

  for (i = 0; i < 2; i++)
  {
    if (i == 0)
      attr = COLOR_PAIR(CPAIR_CYAN);
    else
      attr = COLOR_PAIR(CPAIR_RED);

    sprintf(buf, "%c %d/%d",
            (i == 0) ? '@' : '&',
            wp->state[99 + i], wp->victory_condition);

    attron(attr);
    mvaddstr(PLAYER_OFFSET_Y + 8 - i * 8,
             PLAYER_OFFSET_X + 2,
             buf);
    attroff(attr);
  }

  return 0;
}

static int
loop_output_monster(world *wp)
{
  int i;
  int grid_from;
  int grid_to;
  int grid_from_x;
  int grid_from_y;
  int grid_to_x;
  int grid_to_y;
  int attr;
  int attr_friendly;
  int attr_enemy;
  int score;
  char buf[256];

  if (wp == NULL)
    return 1;

  if (wp->mp->layer_top < 1)
    return 0;

  score = wp->mp->score[wp->mp->layer_top];
  if (wp->mp->turn_player[wp->mp->layer_top] == 0)
  {
    attr_friendly = COLOR_PAIR(CPAIR_CYAN);
    attr_enemy = COLOR_PAIR(CPAIR_RED);
  }
  else
  {
    attr_friendly = COLOR_PAIR(CPAIR_RED);
    attr_enemy = COLOR_PAIR(CPAIR_CYAN);
    score *= -1;
  }

  attr = attr_friendly;

  attron(attr);
  sprintf(buf, "score: %d", score);
  mvaddstr(MONSTER_OFFSET_Y + 0, MONSTER_OFFSET_X + 0, buf);
  attroff(attr);

  /* assumes ASCII code*/
  for (i = wp->mp->layer_top; i >= 1; i--)
  {
    grid_from = wp->mp->grid_from[wp->mp->layer_top][i];
    grid_to = wp->mp->grid_to[wp->mp->layer_top][i];
    if ((grid_from >= 0) && (grid_from < 81)
        && (grid_to >= 0) && (grid_to < 81))
    {
      grid_from_x = grid_from % 9;
      grid_from_y = grid_from / 9;
      grid_to_x = grid_to % 9;
      grid_to_y = grid_to / 9;
      sprintf(buf, "%c %c%c - %c%c",
              (wp->mp->turn_player[i] == 0) ? '@' : '&',
              'a' + grid_from_x, '1' + grid_from_y,
              'a' + grid_to_x, '1' + grid_to_y);
    }
    else
    {
      break;
    }

    if ((wp->mp->layer_top - i) % 2 == 0)
      attr = attr_friendly;
    else
      attr = attr_enemy;

    attron(attr);
    mvaddstr(MONSTER_OFFSET_Y + 1 + wp->mp->layer_top - i,
             MONSTER_OFFSET_X + 0,
             buf);
    attroff(attr);
  }

  return 0;
}

static int
message_before_game(world *wp)
{
  int attr;
  char buf[256];

  if (wp == NULL)
    return 1;

  attr = COLOR_PAIR(CPAIR_WHITE);

  attron(attr);

  sprintf(buf, "blue:              %c%s%c",
          (wp->option_item == OPTION_ITEM_BLUE) ? '<' : ' ',
          (wp->player[0] == 0)
          ? " human  "
          : "computer",
          (wp->option_item == OPTION_ITEM_BLUE) ? '>' : ' ');
  mvaddstr(MESSAGE_OFFSET_Y + 0,
           MESSAGE_OFFSET_X + 0,
           buf);

  sprintf(buf, "red:               %c%s%c",
          (wp->option_item == OPTION_ITEM_RED) ? '<' : ' ',
          (wp->player[1] == 0)
          ? " human  "
          : "computer",
          (wp->option_item == OPTION_ITEM_RED) ? '>' : ' ');
  mvaddstr(MESSAGE_OFFSET_Y + 1,
           MESSAGE_OFFSET_X + 0,
           buf);

  sprintf(buf, "victory condition: %c%d%c",
          (wp->option_item == OPTION_ITEM_VICTORY_CONDITION) ? '<' : ' ',
          wp->victory_condition,
          (wp->option_item == OPTION_ITEM_VICTORY_CONDITION) ? '>' : ' ');
  mvaddstr(MESSAGE_OFFSET_Y + 2,
           MESSAGE_OFFSET_X + 0,
           buf);

  mvaddstr(MESSAGE_OFFSET_Y + 4,
           MESSAGE_OFFSET_X + 0,
           "([up/down] choose item, [left/right] change value)");
  mvaddstr(MESSAGE_OFFSET_Y + 5,
           MESSAGE_OFFSET_X + 0,
           "([5/.] start game, [Q] quit)");

  attroff(attr);

  return 0;
}

static int
message_after_game(world *wp)
{
  int attr;

  if (wp == NULL)
    return 1;

  attr = COLOR_PAIR(CPAIR_WHITE);

  attron(attr);

  if ((wp->state[99] >= wp->victory_condition)
      && (wp->state[100] < wp->victory_condition))
  {
    mvaddstr(MESSAGE_OFFSET_Y + 0,
             MESSAGE_OFFSET_X + 0,
             "blue wins");
  }
  else if ((wp->state[99] < wp->victory_condition)
      && (wp->state[100] >= wp->victory_condition))
  {
    mvaddstr(MESSAGE_OFFSET_Y + 0,
             MESSAGE_OFFSET_X + 0,
             "red wins");
  }
  else
  {
    mvaddstr(MESSAGE_OFFSET_Y + 0,
             MESSAGE_OFFSET_X + 0,
             "draw");
  }

  mvaddstr(MESSAGE_OFFSET_Y + 1,
           MESSAGE_OFFSET_X + 0,
           "([R] reset, [Q] quit)");

  attroff(attr);

  return 0;
}

static int
loop_output_message(world *wp)
{
  if (wp == NULL)
    return 1;

  switch (wp->mode)
  {
  case MODE_BEFORE_GAME:
    message_before_game(wp);
    break;
  case MODE_PLAYING:
    break;
  case MODE_AFTER_GAME:
    message_after_game(wp);
    break;
  default:
    fprintf(stderr, "loop_output_message: unknown mode (%d)\n", wp->mode);
    return 1;
    break;
  }

  return 0;
}

int
loop_output(world *wp)
{
  if (wp == NULL)
  {
    fprintf(stderr, "loop_output: wp is NULL\n");
    return 1;
  }

  if (!state_is_sane(wp->state))
  {
    fprintf(stderr, "loop_output: wp->state is not sane\n");
    return 1;
  }

  erase();

  if (wp->cheat_peek_monster)
  {
    loop_output_monster(wp);
  }
  loop_output_player(wp);
  loop_output_message(wp);
  loop_output_board(wp);

  move(BOARD_OFFSET_Y + 8 - wp->cursor_y, BOARD_OFFSET_X + wp->cursor_x);
  refresh();

  return 0;
}
