#include <stdio.h>
#include <curses.h>
/* memcpy */
#include <string.h>

#include "world.h"
#include "rule.h"
#include "think.h"
#include "mode.h"

#include "loop-process.h"

int
loop_process(world *wp)
{
  if (wp == NULL)
  {
    fprintf(stderr, "loop_process: wp is NULL\n");
    return 1;
  }

  if (!state_is_sane(wp->state))
  {
    fprintf(stderr, "loop_process: wp->state is not sane\n");
    return 1;
  }

  if (wp->mode != MODE_PLAYING)
  {
    return 0;
  }

  if (!(wp->waiting_for_move))
  {
    wp->waiting_for_move = 1;
    return 0;
  }

  while (1)
  { 
    if (wp->player[wp->turn_player] > 0)
    {
      monster_init(wp, wp->player[wp->turn_player]);
      monster_think(wp->mp, wp->mp->layer_top);

      wp->grid_from = wp->mp->grid_from[wp->mp->layer_top][wp->mp->layer_top];
      wp->grid_to = wp->mp->grid_to[wp->mp->layer_top][wp->mp->layer_top];

      if (wp->grid_to < 0)
      {
        fprintf(stderr, "loop_process: computer stops thinking\n");
        return 1;
      }
    }

    if (wp->grid_to >= 0)
    {
      if (move_is_legal(wp->state, wp->turn_player,
                        wp->grid_from, wp->grid_to,
                        wp->victory_condition))
      {
        remove_corpse(wp->state);
        move_piece(wp->state, wp->grid_from, wp->grid_to);
        capture_piece(wp->state, wp->grid_to);
      }
      else
      {
        fprintf(stderr, "loop_process: move is illegal ((%d) - (%d))\n",
                wp->grid_from, wp->grid_to);
      }

      wp->last_move = wp->grid_to;

      wp->grid_from = -1;
      wp->grid_to = -1;

      if (wp->turn_player == 0)
        wp->turn_player = 1;
      else
        wp->turn_player = 0;

      wp->waiting_for_move = 0;
    }

    if ((wp->state[99] < wp->victory_condition)
        || (wp->state[100] < wp->victory_condition))
    {
      wp->mode = MODE_AFTER_GAME;
      break;
    }

    if ((!(wp->waiting_for_move)) || (wp->player[wp->turn_player] <= 0))
      break;
  }

  return 0;
}
