#include <stdio.h>
/* strcmp */
#include <string.h>

#include "config.h"

#include "world.h"
#include "init-curses.h"
#include "loop.h"
#include "rule.h"
#include "jkiss32.h"

#define VERSION "0.2"

static int do_version(void);
static int do_help(void);

static int
do_version(void)
{
  printf("hth (Hasami Tenbin Hashi-otoshi)\n");
  printf("version %s\n", VERSION);
  printf("Copyright (C) 2013 Oohara Yuuma <oohara@libra.interq.or.jp>\n");
  printf("This program is free software;\n");
  printf("see the file LICENSE in the source code for details.\n");
  printf("hth comes with NO WARRANTY, to the extent "
         "permitted by law.\n");

  return 0;
}

static int
do_help(void)
{
  printf("Usage: hth [options]\n");
  printf("Hasami Tenbin Hashi-otoshi, a board game\n");
#ifdef USE_BUILT_IN_RAND
  printf("uses built-in RNG\n");
#else /* not USE_BUILT_IN_RAND */
  printf("uses library rand()\n");
#endif /* not USE_BUILT_IN_RAND */
  printf("Options:\n");
  printf("--help                    print this message\n");
  printf("--version                 print version information\n");
  printf("--cheat-peek              show what computer player thinks\n");

  return 0;
}

int
main(int argc, char *argv[])
{
  int i;
  int flag_version;
  int flag_help;
  int flag_peek_monster;
  world *wp = NULL;

  flag_help = 0;
  flag_version = 0;
  flag_peek_monster = 0;
  for (i = 1; i < argc; i++)
  {
    if (!strcmp(argv[i], "--help"))
    {
      flag_help = 1;
    }
    else if (!strcmp(argv[i], "--version"))
    {
      flag_version = 1;
    }
    else if (!strcmp(argv[i], "--cheat-peek"))
    {
      flag_peek_monster = 1;
    }
    else
    {
      fprintf(stderr, "main: unknown option: %s\n", argv[i]);
      fprintf(stderr, "Type %s --help for the list of options.\n", argv[0]);
      return 1;
    }
  }

  if (flag_version)
    do_version();
  if (flag_help)
    do_help();
  if ((flag_help) || (flag_version))
    return 0;

  if (initialize_rand() != 0)
  {
    fprintf(stderr, "main: initialize_rand failed\n");
    return 1;
  }

  wp = world_new();
  if (wp == NULL)
  {
    fprintf(stderr, "main: world_new() failed\n");
    return 1;
  }

  if (init_curses(wp) != 0)
  {
    fprintf(stderr, "main: init_curses() failed\n");
    world_delete(wp);
    wp = NULL;
    return 1;
  }

  wp->cheat_peek_monster = flag_peek_monster;

  state_reset(wp->state);

  main_loop(wp);

  world_delete(wp);
  wp = NULL;

  return 0;
}
